/*
 *   GtkAudioScale
 *    
 *   Copyright (C) Robert Ham 2002 (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <float.h>

#include "gtkaudioscale/audioscale.h"
#include "gtkaudioscale/haudioscale.h"

#define _(x) x

static void gtk_haudio_scale_class_init (GtkHAudioScaleClass * class);

static GtkAudioScaleClass *parent_class = NULL;

GType
gtk_haudio_scale_get_type ()
{
  static GType haudio_scale_type = 0;
  
  if (!haudio_scale_type)
    {
      static const GTypeInfo haudio_scale_info =
      {
        sizeof (GtkHAudioScaleClass),
        NULL,
        NULL,
        (GClassInitFunc) gtk_haudio_scale_class_init,
        NULL,
        NULL,
        sizeof (GtkHAudioScale),
        0,
        NULL
      };

      haudio_scale_type = g_type_register_static (GTK_TYPE_AUDIO_SCALE,
                                                  "GtkHAudioScale",
                                                  &haudio_scale_info,
                                                  0);
    }
  
  return haudio_scale_type;
}

static void
gtk_haudio_scale_class_init (GtkHAudioScaleClass * class)
{
  GObjectClass * gobject_class;
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
      
  parent_class = g_type_class_peek_parent (class);
  gobject_class = G_OBJECT_CLASS (class);
  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;
  
}

GtkWidget *
gtk_haudio_scale_new ()
{
  return gtk_haudio_scale_new_with_adjustment (NULL, FALSE);
}

GtkWidget *
gtk_haudio_scale_new_with_adjustment (GtkAdjustment * adjustment, gboolean logarithmic)
{
  GtkAdjustment * adj;
  GtkStyle * zoom_upper_style;
  GtkStyle * zoom_lower_style;
  GtkHAudioScale * hscale;
  
  if (!adjustment)
    adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.0 + DBL_EPSILON,
                                              1.0,
                                              1.0 + DBL_EPSILON*2,
                                              DBL_EPSILON,
                                              DBL_EPSILON,
                                              DBL_EPSILON));
  else
    adj = adjustment;
  
  g_object_ref (G_OBJECT (adj));
  
  zoom_upper_style = gtk_style_new ();
  zoom_lower_style = gtk_style_new ();
  
  hscale = g_object_new (GTK_TYPE_HAUDIO_SCALE,
                         "logarithmic", logarithmic,
                         "adjustment", adj,
                         "zoom_upper_style", zoom_upper_style,
                         "zoom_lower_style", zoom_lower_style,
                         "orientation", GTK_ORIENTATION_HORIZONTAL,
                         NULL);
  
  return GTK_WIDGET (hscale);
}

