/*
 *   jack-ladspa-host
 *    
 *   Copyright (C) Robert Ham 2002 (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ac_config.h"

#include <gtk/gtk.h>
#define _GNU_SOURCE
#include <getopt.h>
#undef _GNU_SOURCE
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <glib.h>

#ifdef HAVE_LADCCA
#include <ladcca/ladcca.h>
#endif

#ifdef HAVE_GNOME
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#endif

#include "control_message.h"
#include "jack_rack.h"
#include "plugin.h"
#include "ac_config.h"
#include "callbacks.h"
#include "plugin_manager.h"

jack_rack_t * global_jack_rack;

#ifdef HAVE_LADCCA
cca_client_t * global_cca_client;
#endif

void print_help () {
  printf("%s version %s\n", PACKAGE_NAME, PACKAGE_VERSION);
  printf("Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)\n");
  printf("\n");
  printf("This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it\n");
  printf("under the terms of the GNU General Public License, version 2 or later.  See\n");
  printf("the COPYING file that came with this software for details.\n");
  printf("\n");
#ifdef HAVE_LADCCA
  printf("Compiled with JACK %s, LADCCA %s\n", JACK_VERSION, LADCCA_VERSION);
#else
  printf("Compiled with JACK %s\n", JACK_VERSION);
#endif
  printf("\n");
  printf(" -h, --help                  Display this help info\n");
  printf(" -p, --pid-name              Use the pid in the JACK client name\n");
  printf(" -s, --string-name <string>  Use <string> in the JACK client name\n");
#ifdef HAVE_JACK_SET_SERVER_DIR
  printf(" -D, --tmpdir <dir>          Tell JACK to use <dir> for its temporary files\n");
#endif
  printf(" -i, --input                 Connected inputs to the first two physical capture ports\n");
  printf(" -o, --output                Connected outputs to the first two physical playback ports\n");
  printf("\n");
}

int main (int argc, char ** argv) {
  char * client_name = NULL;
  int connect_inputs = 0;
  int connect_outputs = 0;
  int opt;

#ifdef HAVE_JACK_SET_SERVER_DIR
  const char * options = "hps:D:io";
#else
  const char * options = "hps:io";
#endif /* HAVE_JACK_SET_SERVER_DIR */

  struct option long_options[] = {
    { "help", 0, NULL, 'h' },
    { "pid-name", 0, NULL, 'p' },
    { "string-name", 1, NULL, 's' },
#ifdef HAVE_JACK_SET_SERVER_DIR
    { "tmpdir", 1, NULL, 'D' },
#endif
    { "input", 0, NULL, 'i' },
    { "output", 0, NULL, 'o' },
    { 0, 0, 0, 0 }
  };

#ifdef HAVE_LADCCA
  char * ladcca_client_name = NULL;
  cca_args_t * cca_args;
  cca_event_t * cca_event;
#endif  

  /* for printing out memory usage at the end */
/*  g_mem_set_vtable (glib_mem_profiler_table); */

#ifdef HAVE_LADCCA
  cca_args = cca_extract_args (&argc, &argv);
#endif  

  gtk_set_locale ();
  gtk_init (&argc, &argv);

#ifdef HAVE_GNOME
  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                      argc, argv, NULL);
#endif  
  
  while ((opt = getopt_long (argc, argv, options, long_options, NULL)) != -1) {
    switch (opt) {
      case 'h':
        print_help ();
        exit (0);
        break;
      case 's':
        if (!client_name) client_name = g_strdup_printf ("jack_rack_%s", optarg);
#ifdef HAVE_LADCCA
        if (!ladcca_client_name) ladcca_client_name = g_strdup_printf ("JACK Rack (%s)", optarg);
#endif
        break;
      case 'p':
        if (!client_name) client_name = g_strdup_printf ("jack_rack_%d", getpid());
#ifdef HAVE_LADCCA
        if (!ladcca_client_name) ladcca_client_name = g_strdup_printf ("JACK Rack (%d)", getpid());
#endif
        break;
#ifdef HAVE_JACK_SET_SERVER_DIR
      case 'D':
        jack_set_server_dir (optarg);
        break;
#endif
      case 'i':
        connect_inputs = 1;
        break;
      case 'o':
        connect_outputs = 1;
        break;
      case ':':
        fprintf (stderr, "missing option parameter\n");
        print_help ();
        exit (1);
        break;
      case '?':
        print_help ();
        exit (1);
        break;
    }
  }
  

#ifdef HAVE_LADCCA
  if (!ladcca_client_name)
    {
      ladcca_client_name = g_strdup ("JACK Rack");
    }

  global_cca_client = cca_init (cca_args, "JACK Rack" , CCA_Use_Jack|CCA_Config_File, CCA_PROTOCOL (1,0));

  if (cca_enabled (global_cca_client))
    {
      cca_event = cca_event_new_with_type (CCA_Client_Name);
      cca_event_set_string (cca_event, ladcca_client_name);
      cca_send_event (global_cca_client, cca_event);
    }
#endif

  if (!client_name) {
        client_name = g_strdup ("jack_rack");
  }

  global_jack_rack = jack_rack_new (client_name);
  g_free (client_name);
  
  
  jack_on_shutdown (global_jack_rack->process_info->jack_client,
                    jack_shutdown_callback,
                    &global_jack_rack);
  
  jack_activate (global_jack_rack->process_info->jack_client);
  
  gtk_main ();
  
  jack_client_close (global_jack_rack->process_info->jack_client);
  
  jack_rack_destroy (global_jack_rack);
  
  
/*  g_mem_profile (); */
  
  return 0;
}

